import sys
from itertools import chain

import pymysql

class ConnectClassDriverLoadError(Exception):pass

class ChooseDriver():
    def __init__(self) -> None:
        self.fun=None
    def __call__(self, fun,*args, **kwargs):
        self.fun=fun
        fun=self.choose
        return fun
    def choose(self,obj) -> object:
        try:
            return getattr(obj,obj.driver)()
        except Exception as e:
            raise ConnectClassDriverLoadError(
                'Connection error, the driver cannot be found or is not supported: %s' %
                (e)
            )

class Connection():
    def __init__(self,driver,*args,**kwargs) -> None:
        self.driver=driver.lower()
        self.args=args
        self.kwargs=kwargs
        self.conn=None
        self.fun=None

    def __call__(self, fun,*args, **kwargs):
        self.fun=fun
        self.create_conn(self)
        fun=self.result
        return fun
    def __ioc__(self) -> None:pass

    def result(self) -> object:return self.conn

    @ChooseDriver()
    def create_conn(self) -> object:pass

    def mysql(self) -> object:
            self.conn=pymysql.connect(
                host=self.kwargs['host'],
                user=self.kwargs['user'],
                password=self.kwargs['password'],
                port=self.kwargs['port'],
                database=self.kwargs['database']
            )
            sys.modules[str(self.fun).split(" ")[1].split('.')[0]+"."+self.__class__.__module__+'.'+self.__class__.__name__+'-mysql']=self.conn

    def oracle(self) -> object:pass

    def sqlLite(self) -> object:pass

    @classmethod
    def connect(self,fun):
        def _w(obj,driver,*args,**kwargs):
            if not sys.modules.__contains__(str(obj.cla).split(' ')[1].split('.')[0]+'.'+self.__module__+'.'+self.__name__+'-'+driver):
                raise ConnectClassDriverLoadError(
                    'Connection information not found: %s' % (str(obj.cla).split(' ')[1].split('.')[0]+'.'+self.__module__+'.'+self.__name__+'-'+driver)
                )
            conn = sys.modules[str(obj.cla).split(' ')[1].split('.')[0]+'.'+self.__module__+'.'+self.__name__+'-'+driver]
            return conn
        return _w

class ExecSQL():
    def __init__(self):pass
    def __call__(self, fun,*args, **kwargs):
        fun=self.exec
        return fun
    def exec(self,fun,sql) ->any:
        if fun.driver=='mysql':
            if isinstance(sql,str):
                return self.execMySql(fun, sql, fun.driver)
            else:
                return self.execMySql1(fun,sql,fun.driver)
        if fun.driver=='oracle':
            pass
        if fun.driver=='sqllite':
            pass

    def execMySql(self,fun,sql,driver) -> any:
        returnType = fun.cla.__annotations__['return']
        conn = fun.connect_pool(driver)
        cur_type=None
        try:
            if returnType.__name__ == 'dict':
                cur_type =pymysql.cursors.DictCursor
            cursor=conn.cursor(cursor=cur_type)
            reint=cursor.execute(sql)
            conn.commit()
            result=cursor.fetchall()
            if returnType.__name__ == 'list':
                return list(chain.from_iterable(result))
            if returnType.__name__ == 'int':
                return reint
            if returnType.__name__ == 'bool':
                if reint>0:
                    return True
                else:
                    return False
            return result
        except Exception as e:
            conn.rollback()
            raise e
    def execMySql1(self,fun,parm,driver) -> any:
        returnType = fun.cla.__annotations__['return']
        cur_type = None
        sql,data=parm[0],parm[1]
        conn = fun.connect_pool(driver)
        try:
            if returnType.__name__ == 'dict':
                cur_type =pymysql.cursors.DictCursor
            cursor=conn.cursor(cursor=cur_type)
            reint=cursor.executemany(sql,data)
            conn.commit()
            result=cursor.fetchall()
            if returnType.__name__ == 'list':
                return list(chain.from_iterable(result))
            if returnType.__name__ == 'dict':
                return result
            if returnType.__name__ == 'bool':
                if reint>0:
                    return True
                else:
                    return False
            return reint
        except Exception as e:
            conn.rollback()
            raise e















import sys

from annosSQL.test.tesys import Ut001

u=Ut001()
u.handler()
data1=u.allUser()
data2=u.iduser(1)
# r=u.inserts("cai,0987654321,9")
print(data2)
print(data1)
# print(sys.modules)
# d2=u.Tinsert([('qqq','qewqwrqr',19),('jjjj','1232141',88)])
# print(d2)


import sys

from annosSQL.Donos.doconn import Connection


class InterfaceClassSyntaxError(Exception):
    """Raised when adding API names to symbol that already has API names."""
    pass

class Interface():
    def __init__(self, *args, **kwargs) -> any:
        self.handlerNUm=0
        self.connNUm=0

    def __call__(self, fun, *args, **kwargs) -> type:
        filter_k = ['__module__', '__dict__', '__weakref__', '__doc__']
        for i in vars(fun).keys():
            if i in '__init__':
                raise InterfaceClassSyntaxError(
                    'Use @Interface annosSQL is not require realization %s() method.' %
                    (i))
            if i not in filter_k and '__self__' in dir(vars(fun).get(i)) and isinstance(vars(fun).get(i).__self__,Connection):
                vars(getattr(fun, i)).setdefault("fun", fun)
                self.connNUm+=1
                continue
            if i not in filter_k and '__self__' in dir(vars(fun).get(i)) and isinstance(vars(fun).get(i).__self__,Handler):
                vars(getattr(fun, i)).setdefault("fun", fun)
                self.handlerNUm+=1
                continue
            if i not in filter_k and not self.isEmptyFunction(getattr(fun, i)):
                raise InterfaceClassSyntaxError(
                    'The method %s.%s() should be an empty function.' %
                    (fun.__name__, i)
                )
            if i not in filter_k and not dict(getattr(fun, i).__annotations__).__contains__("return"):
                raise InterfaceClassSyntaxError(
                    'This method %s.%s() return type not defined,Correct definition: def %s(parameter:any): -> type:pass' %
                    (fun.__name__, i, i)
                )
        if self.handlerNUm<=0 or self.connNUm<=0:
            raise InterfaceClassSyntaxError(
                '%s class not found  handler method for annotation @handler or @Connection.' %
                (fun.__name__)
            )
        return fun

    @property
    def empty(self) -> None:pass
    def isEmptyFunction(self, func):
        e = lambda: None
        return func.__code__.co_code == e.__code__.co_code

class Handler():
    def __init__(self, *args, **kwargs) -> None:
        self.fun = None
    def __call__(self, fun, *args, **kwargs) -> object:
        fun.__globals__['Handler'] = fun.__name__
        fun = self.handler
        self.fun = fun
        return fun

    def __class__(self):return self
    def handler(self):
        filter_k = ['__module__', '__dict__', '__weakref__', '__doc__']
        try:
            cla=vars(self.fun)['fun']
            k_list=list(vars(cla).keys())
            [k_list.remove(i) for i in filter_k]
            for k in k_list:
                if not isinstance(vars(cla).get(k).__self__,Handler) and not isinstance(vars(cla).get(k).__self__,Connection):
                    z=vars(cla).get(k).__self__
                    _tmp=sys.modules[cla.__name__+'-'+z.__class__.__module__+'-'+z.__class__.__name__]
                    l='''cla.%s = _tmp['%s'].sqlHandler''' % (k, k)
                    exec(l)
            return cla
        except Exception as e:
            raise InterfaceClassSyntaxError(
                'Interface type not found %s. @Interface' % (e)
            )

class IOCHandler():
    def __init__(self) -> None:pass
    def __call__(self, fun,*args, **kwargs) -> object:
        fun=self.ioc_handler
        return fun
    def ioc_handler(self,obj):
        if not dict(sys.modules).__contains__(str(obj.cla).split(' ')[1].split('.')[0]+'-'+obj.__class__.__module__ + '-' + obj.__class__.__name__):
            # sys.modules[obj.__class__.__module__ + '-' + obj.__class__.__name__] = {}
            sys.modules[str(obj.cla).split(' ')[1].split('.')[0]+'-'+obj.__class__.__module__ + '-' + obj.__class__.__name__]={}
        # tmp = sys.modules[obj.__class__.__module__ + '-' + obj.__class__.__name__]
        # tmp[obj.cla.__name__] = obj
        # sys.modules[obj.__class__.__module__ + '-' + obj.__class__.__name__] = tmp
        tmp2=sys.modules[str(obj.cla).split(' ')[1].split('.')[0]+'-'+obj.__class__.__module__ + '-' + obj.__class__.__name__]
        tmp2[obj.cla.__name__] = obj
        sys.modules[str(obj.cla).split(' ')[1].split('.')[0]+'-'+obj.__class__.__module__ + '-' + obj.__class__.__name__] = tmp2






import pathlib
from setuptools import setup

here = pathlib.Path(__file__).parent.resolve()

# 发布在pip上的说明文档
long_description = (here / 'README.md').read_text(encoding='utf-8')

setup(
    name='annosSQL', # 包名
    version='1.0.2', # 包版本
    description='python for mysql operation', # 包简单描述
    long_description=long_description, # 包长描述
    long_description_content_type='text/markdown', # 包长描述格式
    url='https://gitee.com/chx516/annosSQL',  # 包地址
    author='czh', # 作者
    author_email='utowe@qq.com', # 作者邮件
    license='MIT', # 协议
    keywords='python annos', # 关键词
    packages=[ # 公开的子包
        'Donos',
        'Innos',
        'test',
    ],
    install_requires=['pymysql'], # 依赖
    python_requires='>=3' # python版本依赖
)



import copy

from annosSQL.Innos.Interface import IOCHandler
from annosSQL.Donos.docache import CacheHandler
from annosSQL.Donos.doconn import Connection, ExecSQL


class ContextParameterError(Exception):pass
class SQLSyntaxAndParameterError(Exception):pass


class CheckUpSql():
    def __init__(self):pass
    @classmethod
    def SqlAllhandler(self,sql) -> str:return sql
    @classmethod
    def ASqlhandler(self,sql) -> str:
        table=sql.split(" ")[1]
        sql='select * from '+table
        return sql
    @classmethod
    def ISqlhandler(self,sql,p,args,kwargs) -> str:
        if not dict(p).__contains__("S") or p['S'].__name__!='str':
            raise SQLSyntaxAndParameterError(
                'Syntax error or parameter error: %s' % (p)
            )
        sql=str(sql)
        l=sql.index('S{')
        h=sql.index('}')
        step=sql[l+2:h]
        value=str(args[0].split(',')).replace('[',"").replace(']',"")
        if sql.find('[')!=-1:
            j=sql[sql.find('[')+1:sql.find(']')]
            sql=sql[:sql.find('[')]+sql[sql.find(']')+1:]
            j=j.split(',')
            ap=list(args[0].split(step))
            for i in j:
                ap[int(i) - 1] = int(ap[int(i) - 1])
            value=str(ap).replace('[',"").replace(']',"")
        return sql[:l]+value+sql[h+1:]

    @classmethod
    def LSqlhandler(self,sql,p,args,kwargs):
        if not dict(p).__contains__("L") or p['L'].__name__ != 'list':
            raise SQLSyntaxAndParameterError(
                'Syntax error or parameter error: %s' % (p)
            )
        sql = str(sql)
        l = sql.index('L{')
        h = sql.index('}')
        date=str(args[0]).replace('[', "").replace(']', "")
        return sql[:l] + date + sql[h + 1:]
    @classmethod
    def ISSqlhandler(self,sql,p,args,kwargs):
        if not dict(p).__contains__("T") or p['T'].__name__ != 'list' and p['T'].__name__ != 'tuple':
            raise SQLSyntaxAndParameterError(
                'Syntax error or parameter error: %s' % (p)
            )
        sql=str(sql).replace("T{","").replace("}","")
        data=None
        if len(args)!=0:
            data=args[0]
        if len(kwargs)!=0:
            data=kwargs[list(p.keys())[0]]
        return (sql,data)

class SqlGenerate():
    def __init__(self) -> None:pass

    def __call__(self, fun,*args, **kwargs):
        fun=self.sqlhandler
        return fun
    def checkup(self):
        def _w(obj,sql,p,args,kwargs):
            if len(p)==0 and sql.find("#{")==-1 and sql.find("}")  ==-1 and not 'A(select)'.lower() in sql.lower():
                return CheckUpSql.SqlAllhandler(sql)
            if len(p)==0 and 'A(select)'.lower() in sql.lower():
                return CheckUpSql.ASqlhandler(sql)
            if 'insert' in sql.lower() and 'S{' in sql:
                return CheckUpSql.ISqlhandler(sql,p,args,kwargs)
            if 'L{}' in sql:
                return CheckUpSql.LSqlhandler(sql,p,args,kwargs)
            if dict(p)[list(p.keys())[0]].__name__ in ['list','tuple'] and 'T{' and '%s' in sql:
                return CheckUpSql.ISSqlhandler(sql,p,args,kwargs)
            return self(obj,sql, p, args, kwargs)
        return _w

    @checkup
    def sqlhandler(self,sql,p,args,kwargs) -> str:
        parm=p.keys()
        key=list(p.keys())
        if  not tuple(args) and not  dict(kwargs):
            raise ContextParameterError(
                'Context parameter error: %s or %s' %
                (str(args),str(kwargs))
            )

        if len(parm)==len(args):
            parm=[i for i in args]
        if len(parm)==len(kwargs):
            parm=kwargs
        if len(parm)==len(args)+len(kwargs):
            T=[]
            [T.append(i) for i in args if args]
            [T.append(kwargs[i]) for i in kwargs]
            parm=T
        for i in range(0,len(parm)):
            if p[key[i]].__name__== "str":
                sql = str(sql).replace('{%s}' % (i + 1), "'{parm[%s]}'" % (i))
            else:
                sql=str(sql).replace('{%s}' % (i+1), '{parm[%s]}' % (i))
        sql=sql.format(parm=parm).replace("#",'')
        return sql

class TypeSQL():
    def __init__(self):pass
    def __call__(self, fun,*args, **kwargs):
        fun=self.returnType
        return fun
    def returnType(self,sql) -> object:
        if 'select' in sql.lower():
            return "select"
        if 'insert' in sql.lower():
            return "insert"
        if 'update' in sql.lower():
            return "update"
        if 'delete' in sql.lower():
            return 'delete'
        return 'other'

class execute():
    def __init__(self, sql, driver='mysql',cacheDriver=dict,*args, **kwargs) -> any:
        self.sql = sql
        self.sqlType = None
        self.driver=driver.lower()
        self.cla=None
        self.cacheSize=10 #空间大小
        self.cacheBit=0 #当前占用空间
        self.access=0  #最近访问的缓存空间
        self.accessNot=0 #最久访问
        self.cache={}  #缓存空间
        # self.logicTable=[]  #缓存逻辑表
        self.physicsTable=[] #缓存地址表
        self.id_cursor=0
    def __call__(self, fun,*args, **kwargs) -> type:
        self.cla=fun

        # print(fun.__annotations__)
        fun=self.fun
        self.__ioc__(self)
        return fun

    @IOCHandler()
    def __ioc__(self) -> None:pass
    def fun(self) -> any:pass
    def result(self, *args, **kwargs) -> any:return self

    def sqlHandler(self,*args,**kwargs) -> result:
        self.sqlType=self.TypeHandler(self.sql)
        parm=list(self.cla.__annotations__.keys())
        p = copy.deepcopy(self.cla.__annotations__)
        parm_count=len(args)+len(kwargs)
        if dict(self.cla.__annotations__).__contains__('return'):
            parm.remove('return')
            del p['return']
        if len(parm) != parm_count:
            raise ContextParameterError(
                'The function expects to get parameter %s and now gets %s' %
                (str(len(parm)),str(parm_count))
            )
        currentSql=self.syntaxHandler(self.sql,p,args,kwargs)
        data=self.cache_pool(self,currentSql)

        return data
    @SqlGenerate()
    def syntaxHandler(self) -> str:pass

    @Connection.connect
    def connect_pool(self,driver) -> any: pass

    @CacheHandler()
    def cache_pool(self,fun,sql) -> any:pass

    @ExecSQL()
    def execSql(self,sql) -> any:pass

    @TypeSQL()
    def TypeHandler(self,sql) -> str:pass

    def mapping(self, result): return result





